package com.jz.agent.client.service;


import com.google.gson.Gson;
import com.jz.agent.client.DmpDsAgentClient;
import com.jz.dmp.agent.DmpAgentResult;
import com.jz.dmp.agent.ResultCode;
import com.jz.dmp.modules.model.DmpAgentDatasourceInfo;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class DmpPostgreSqlAgentClientService implements DmpDsAgentClient {
	
	private Gson gson = new Gson();
	private static Logger logger = LoggerFactory.getLogger(DmpPostgreSqlAgentClientService.class);

	@SuppressWarnings("unused")
	public DmpAgentResult testConnect(DmpAgentDatasourceInfo ds) {
		Connection conn = null;
		try {
			Class.forName(ds.getDriverClassName());
			conn = DriverManager.getConnection(ds.getJdbcUrl(), ds.getUserName(), ds.getPassword());
			return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(true));
		} catch (Exception e) {
			e.printStackTrace();
			return new DmpAgentResult(ResultCode.EXCEPTION, "POSTGRESQL连通性异常："+e.getMessage());
		} finally {
			try {
				if(conn != null)
				conn.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}

	public DmpAgentResult getTableNameList(DmpAgentDatasourceInfo ds) {
		String sql = "select relname as tabname from pg_class where relkind='r' and relname not like 'pg_%' and relname not like 'sql_%' order by relname";
		Connection conn = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		List<String> list = new ArrayList<String>();
		try {
			Class.forName("org.postgresql.Driver");// "org.postgresql.Driver"
			conn = DriverManager.getConnection(ds.getJdbcUrl(), ds.getUserName(), ds.getPassword());
			// "jdbc:postgresql://192.168.204.102:5432/test", "postgres", "postgres"
			ps = conn.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				list.add(rs.getString(1).toLowerCase());
			}
		} catch (Exception e) {
			e.printStackTrace();
			return new DmpAgentResult(ResultCode.EXCEPTION, "POSTGRESQL获取表列表异常："+e.getMessage());
		} finally {
			if(rs != null)
				try {
					rs.close();
				} catch (SQLException e1) {
					e1.printStackTrace();
				}
			if(ps != null)
				try {
					ps.close();
				} catch (SQLException e1) {
					e1.printStackTrace();
				}
			try {
				if(conn != null)
				conn.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(list));
	}

	public DmpAgentResult getTableColumnList(DmpAgentDatasourceInfo ds, String tableName) {
		String sql = "SELECT col_description(a.attrelid,a.attnum) as comment,format_type(a.atttypid,a.atttypmod) as type,a.attname as name, a.attnotnull as notnull FROM pg_class as c,pg_attribute as a where c.relname = '"
				+ tableName + "' and a.attrelid = c.oid and a.attnum>0";
		Connection conn = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		List<Map<String, Object>> list = new ArrayList<Map<String, Object>>();
		try {
			Class.forName(ds.getDriverClassName());// "org.postgresql.Driver"
			conn = DriverManager.getConnection(ds.getJdbcUrl(), ds.getUserName(), ds.getPassword());
			// "jdbc:postgresql://192.168.204.102:5432/test", "postgres", "postgres"
			ps = conn.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				Map<String, Object> row = new HashMap<String, Object>();
				String type = rs.getString(2).toUpperCase();
				if(StringUtils.isNotBlank(type) && type.indexOf("(") > -1) {
					int idx1 = type.indexOf("(");
					int idx2 = type.indexOf(")");
					String t = type.substring(0, idx1);
					
					String ts = type.substring(idx1+1, idx2);
					if(ts.indexOf(",") > -1) {
						String[] strs = ts.split(",");
						row.put("size", new Integer(strs[0]));
						row.put("scale", new Integer(strs[1]));
					} else {
						row.put("size", new Integer(ts));
						row.put("scale", 0);
					}
					type = t;
				}
				row.put("name", rs.getString(3).toLowerCase());
				row.put("type", type);
				row.put("comment", rs.getString(1));
				list.add(row);
			}
		} catch (Exception e) {
			e.printStackTrace();
			return new DmpAgentResult(ResultCode.EXCEPTION, "POSTGRESQL获取表字段信息异常："+e.getMessage());
		} finally {
			if(rs != null)
				try {
					rs.close();
				} catch (SQLException e1) {
					e1.printStackTrace();
				}
			if(ps != null)
				try {
					ps.close();
				} catch (SQLException e1) {
					e1.printStackTrace();
				}
			try {
				if(conn != null)
				conn.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(list));
	}

	@SuppressWarnings("resource")
	public DmpAgentResult previewData(DmpAgentDatasourceInfo ds, String tableName) {
		Map<String, Object> map = new HashMap<String, Object>();
		String sql = "select * from " + tableName + " limit 5";
		String sql1 = "SELECT col_description(a.attrelid,a.attnum) as comment,format_type(a.atttypid,a.atttypmod) as type,a.attname as name, a.attnotnull as notnull FROM pg_class as c,pg_attribute as a where c.relname = '"
				+ tableName + "' and a.attrelid = c.oid and a.attnum>0";
		Connection conn = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		try {
			Class.forName(ds.getDriverClassName());// "org.postgresql.Driver"
			conn = DriverManager.getConnection(ds.getJdbcUrl(), ds.getUserName(), ds.getPassword());
			// "jdbc:postgresql://192.168.204.102:5432/test", "postgres", "postgres"
			ps = conn.prepareStatement(sql1);
			rs = ps.executeQuery();
			List<String> headerList = new ArrayList<String>();
			while (rs.next()) {
				headerList.add(rs.getString(3).toLowerCase());
			}
			map.put("header", headerList);

			ps = conn.prepareStatement(sql);
			rs = ps.executeQuery();
			List<List<Object>> resultList = new ArrayList<List<Object>>();
			while (rs.next()) {
				List<Object> row = new ArrayList<Object>();
				for (int i = 1; i <= headerList.size(); i++) {
					row.add(rs.getObject(i));
				}
				resultList.add(row);
			}
			map.put("result", resultList);
		} catch (SQLException e) {
			e.printStackTrace();
			return new DmpAgentResult(ResultCode.EXCEPTION, "POSTGRESQL数据预览异常："+e.getMessage());
		} catch (Exception e) {
			e.printStackTrace();
			return new DmpAgentResult(ResultCode.EXCEPTION, "POSTGRESQL数据预览异常："+e.getMessage());
		} finally {
			if(rs != null)
				try {
					rs.close();
				} catch (SQLException e1) {
					e1.printStackTrace();
				}
			if(ps != null)
				try {
					ps.close();
				} catch (SQLException e1) {
					e1.printStackTrace();
				}
			try {
				if(conn != null)
				conn.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(map));
	}

}
