package com.jz.dmp.modules.service.impl;

import com.jz.dmp.modules.dao.DvTaskRuleTDao;
import com.jz.dmp.modules.model.DvTaskRuleT;
import com.jz.dmp.modules.service.DvTaskRuleTService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * 任务与规则关系表(DvTaskRuleT)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-30 15:17:35
 */
@Service("dvTaskRuleTService")
public class DvTaskRuleTServiceImpl implements DvTaskRuleTService {

    @Autowired
    private DvTaskRuleTDao dvTaskRuleTDao;

    /**
     * 通过ID查询单条数据
     *
     * @param taskRuleId 主键
     * @return 实例对象
     */
    @Override
    public DvTaskRuleT queryById(Long taskRuleId) {
        return this.dvTaskRuleTDao.queryById(taskRuleId);
    }

    /**
     * 查询多条数据
     *
     * @param offset 查询起始位置
     * @param limit  查询条数
     * @return 对象列表
     */
    @Override
    public List<DvTaskRuleT> queryAllByLimit(int offset, int limit) {
        return this.dvTaskRuleTDao.queryAllByLimit(offset, limit);
    }

    /**
     * 新增数据
     *
     * @param dvTaskRuleT 实例对象
     * @return 实例对象
     */
    @Override
    public DvTaskRuleT insert(DvTaskRuleT dvTaskRuleT) {
        this.dvTaskRuleTDao.insert(dvTaskRuleT);
        return dvTaskRuleT;
    }

    /**
     * 修改数据
     *
     * @param dvTaskRuleT 实例对象
     * @return 实例对象
     */
    @Override
    public DvTaskRuleT update(DvTaskRuleT dvTaskRuleT) {
        this.dvTaskRuleTDao.update(dvTaskRuleT);
        return this.queryById(dvTaskRuleT.getTaskRuleId());
    }

    /**
     * 通过主键删除数据
     *
     * @param taskRuleId 主键
     * @return 是否成功
     */
    @Override
    public boolean deleteById(Long taskRuleId) {
        return this.dvTaskRuleTDao.deleteById(taskRuleId) > 0;
    }

    /**
     * 保存规则列表信息
     *
     * @param list
     * @throws Exception
     */
    @Override
    public void saveRule(List<DvTaskRuleT> list) throws Exception {
        dvTaskRuleTDao.insertBatch(list);
    }

    @Override
    public List<Long> getTaskRuleIdsList(Integer taskId) throws Exception {
        return dvTaskRuleTDao.getTaskRuleIdsList(taskId);
    }

    /**
     * 批量删除rule信息
     *
     * @param listRules
     * @return
     */
    @Override
    public void delRulesByIds(List<Long> listRules) throws Exception {
        dvTaskRuleTDao.delRulesByIds(listRules);
    }
}